searchData={"items":[{"type":"module","title":"Membrane.VideoCutter","doc":"Membrane element that cuts raw video.\n\nThe element expects each frame to be received in a separate buffer, so the parser\n(`Membrane.Element.RawVideo.Parser`) may be required in a pipeline before\nthe encoder (e.g. when input is read from `Membrane.File.Source`).\n\nTo use this element, specify the desired intervals in the `intervals` options -\n`VideoCutter` will \"filter\" out all frames with timestamps outside of them.","ref":"Membrane.VideoCutter.html"},{"type":"module","title":"Element options - Membrane.VideoCutter","doc":"Passed via struct `t:Membrane.VideoCutter.t/0`\n\n- `intervals`  \n\n  ```\n  [{Membrane.Time.t(), Membrane.Time.t() | :infinity}]\n  ```\n  \n  Default value: `[{0, :infinity}]`  \n  List of intervals of timestamps. The buffer is forwarded when its timestamp belongs to any of the given intervals.\n  The start of the interval is inclusive and the end is exclusive.\n  \n  For example, to cut part starting from 190 ms up to 1530 ms out of the video,\n  the `intervals` should be set to `[{0, Membrane.Time.miliseconds(190)}, {Membrane.Time.miliseconds(1530), :infinity}]`.\n\n\n- `offset`  \n\n  ```\n  Membrane.Time.t()\n  ```\n  \n  Default value: `0`  \n  Offset applied to all cut frames' presentation timestamp (PTS) values. It allows to logically shift the video to express its real starting point.\n  \n  For example, if there are two streams and the second one begins two seconds after the first one,\n  video cutter that processes the second stream should apply a 2sec offset. Offset is applied after cutting phase.","ref":"Membrane.VideoCutter.html#module-element-options"},{"type":"module","title":"Pads - Membrane.VideoCutter","doc":"### `:input`\n\nAccepted formats:\n```\n%RawVideo{aligned: true}\n```\n\nDirection: | `:input`\nAvailability: | `:always`\nFlow control: | `:auto`\n\n\n### `:output`\n\nAccepted formats:\n```\n%RawVideo{aligned: true}\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:auto`","ref":"Membrane.VideoCutter.html#module-pads"},{"type":"function","title":"Membrane.VideoCutter.options/0","doc":"Returns description of options available for this module","ref":"Membrane.VideoCutter.html#options/0"},{"type":"type","title":"Membrane.VideoCutter.t/0","doc":"Struct containing options for `Membrane.VideoCutter`","ref":"Membrane.VideoCutter.html#t:t/0"},{"type":"module","title":"Membrane.VideoMerger","doc":"Membrane element that merges multiple raw videos into one.\n\nThe element expects each frame to be received in a separate buffer, so the parser\n(`Membrane.Element.RawVideo.Parser`) may be required in a pipeline before\nthe merger (e.g. when input is read from `Membrane.File.Source`).\n\nThe element expects to receive frames in order from each input.\n\nCurrently, `VideoMerger` may not be suitable for live merging streams: the element\nawaits for at least one frame from each of the inputs, and forwards the one\nwith the lowest presentation timestamp.","ref":"Membrane.VideoMerger.html"},{"type":"module","title":"Pads - Membrane.VideoMerger","doc":"### `:input`\n\nAccepted formats:\n```\n%RawVideo{aligned: true}\n```\n\nDirection: | `:input`\nAvailability: | `:on_request`\nFlow control: | `:manual`\nDemand unit: | `:buffers`\n\n\n### `:output`\n\nAccepted formats:\n```\n%RawVideo{aligned: true}\n```\n\nDirection: | `:output`\nAvailability: | `:always`\nFlow control: | `:manual`\nDemand unit: | `nil`","ref":"Membrane.VideoMerger.html#module-pads"},{"type":"module","title":"Membrane.VideoCutAndMerge","doc":"Membrane Bin that cuts and merges multiple raw videos into one.\n\nThe bin expects each frame to be received in a separate buffer, so the parser\n(`Membrane.Element.RawVideo.Parser`) may be required in a pipeline before\nthe merger bin (e.g. when input is read from `Membrane.File.Source`).\n\nThe element expects to receive frames in order from each input.\n\nThe bin consists of single `Membrane.VideoMerger` and multiple\n`Membrane.VideoCutter`. Number of elements is constant: cutters are\ncreated at initialization, one for each stream.","ref":"Membrane.VideoCutAndMerge.html"},{"type":"module","title":"Pads - Membrane.VideoCutAndMerge","doc":"### `:input`\n\nAccepted formats:\n```\n%RawVideo{aligned: true}\n```\n\nDirection: | `:input`\nAvailability: | `:on_request`\nPad options:\n\n- `stream`  \n\n  ```\n  Stream.t()\n  ```\n  \n  ***Required***  \n  A stream to cut and merge\n\n\n\n### `:output`\n\nAccepted formats:\n```\n%RawVideo{aligned: true}\n```\n\nDirection: | `:output`\nAvailability: | `:always`","ref":"Membrane.VideoCutAndMerge.html#module-pads"},{"type":"type","title":"Membrane.VideoCutAndMerge.input_pad_opts/0","doc":"Options for pad `:input`","ref":"Membrane.VideoCutAndMerge.html#t:input_pad_opts/0"},{"type":"module","title":"Membrane.VideoCutAndMerge.Stream","doc":"Structure describing video stream to merge by `Membrane.VideoCutAndMerge`","ref":"Membrane.VideoCutAndMerge.Stream.html"},{"type":"module","title":"Fields - Membrane.VideoCutAndMerge.Stream","doc":"- `:intervals` - List of intervals of timestamps that are supposed to be\n  cut and kept from the stream.\n  - `:offset` - Offset applied to all franes' presentation timestamp values.","ref":"Membrane.VideoCutAndMerge.Stream.html#module-fields"},{"type":"type","title":"Membrane.VideoCutAndMerge.Stream.t/0","doc":"","ref":"Membrane.VideoCutAndMerge.Stream.html#t:t/0"},{"type":"extras","title":"Membrane VideoMerger Plugin","doc":"# Membrane VideoMerger Plugin\n\n[![Hex.pm](https://img.shields.io/hexpm/v/membrane_video_merger_plugin.svg)](https://hex.pm/packages/membrane_video_merger_plugin)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/membrane_video_merger_plugin)\n[![CircleCI](https://circleci.com/gh/membraneframework/membrane_video_merger_plugin.svg?style=svg)](https://circleci.com/gh/membraneframework/membrane_video_merger_plugin)\n\nPlugin containing elements for cutting and merging raw video. By using this plugin you can\n  - simply merge multiple video tracks (`VideoMerger`)\n  - cut out specific intervals of video track (`VideoCutter`)\n  - combine chosen parts raw video tracks into one (`VideoCutAndMerge`), for instance by taking\n  the first half of one track and combining it with the second half of another\n\nFor most cases, the `VideoCutAndMerge` bin should be your first choice.\n\nImplementations of the `VideoCutter` and the `VideoMerger` are using presentation timestamps,\nso every `Buffer` should have the `pts` field set.\n\nIt is part of [Membrane Multimedia Framework](https://membraneframework.org).","ref":"readme.html"},{"type":"extras","title":"Installation - Membrane VideoMerger Plugin","doc":"The package can be installed by adding `membrane_video_merger_plugin` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:membrane_video_merger_plugin, \"~> 0.9.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Sample Usage - Membrane VideoMerger Plugin","doc":"Both pipelines will result in creating `/tmp/output.raw` file with the first 5 seconds of the\n`input_1` track and everything but the first 5 seconds of the `input_2` track.\n\n`/tmp/input_1.h264` and `/tmp/input_2.h264` should be an H264 video files.\n\n#","ref":"readme.html#sample-usage"},{"type":"extras","title":"CutAndMerge bin - Membrane VideoMerger Plugin","doc":"```elixir\ndefmodule VideoCutAndMerge.Pipeline do\n  use Membrane.Pipeline\n\n  alias Membrane.VideoCutAndMerge\n  alias Membrane.H264.FFmpeg.{Parser, Decoder}\n  alias Membrane.File.{Sink, Source}\n\n  @impl true\n  def handle_init(_ctx, _options) do\n    stream_1 = %VideoCutAndMerge.Stream{intervals: [{0, Membrane.Time.seconds(5)}]}\n    stream_2 = %VideoCutAndMerge.Stream{intervals: [{Membrane.Time.seconds(5), :infinity}]}\n\n    structure = [\n      child(:cut_and_merge, VideoCutAndMerge)\n      |> child(:sink, %Sink{location: \"/tmp/output.raw\"}),\n\n      child({:file_src, 1}, %Source{chunk_size: 40_960, location: \"/tmp/input_1.h264\"})\n      |> child({:parser, 1}, %Parser{framerate: {30, 1}})\n      |> child({:decoder, 1}, Decoder)\n      |> via_in(Pad.ref(:input, 1), options: [stream: stream_1])\n      |> get_child(:cut_and_merge),\n\n      child({:file_src, 2}, %Source{chunk_size: 40_960, location: \"/tmp/input_2.h264\"})\n      |> child({:parser, 2}, %Parser{framerate: {30, 1}})\n      |> child({:decoder, 2}, Decoder)\n      |> via_in(Pad.ref(:input, 2), options: [stream: stream_2])\n      |> get_child(:cut_and_merge)\n    ]\n\n    {[spec: structure], %{}}\n  end\nend\n```\n\n#","ref":"readme.html#cutandmerge-bin"},{"type":"extras","title":"VideoCutter and VideoMerger - Membrane VideoMerger Plugin","doc":"```elixir\ndefmodule VideoMerger.Pipeline do\n  use Membrane.Pipeline\n\n  alias Membrane.H264.FFmpeg.{Parser, Decoder}\n  alias Membrane.File.{Sink, Source}\n  alias Membrane.{VideoCutter, VideoMerger}\n\n  @impl true\n  def handle_init(_ctx, _options) do\n    structure = [\n      child(:merger, VideoMerger)\n      |> child(:sink, %Sink{location: \"/tmp/output.raw\"),\n\n      child({:file_src, 1}, %Source{chunk_size: 40_960, location: \"/tmp/input_1.h264\"})\n      |> child({:parser, 1}, %Parser{framerate: {30, 1}})\n      |> child({:decoder, 1}, Decoder)\n      |> child({:cutter, 1}, %VideoCutter{intervals: [{0, Membrane.Time.seconds(5)}]})\n      |> via_in(Pad.ref(:input, 1))\n      |> get_child(:merger),\n\n      child({:file_src, 2}, %Source{chunk_size: 40_960, location: \"/tmp/input_2.h264\"})\n      |> child({:parser, 2}, %Parser{framerate: {30, 1}})\n      |> child({:decoder, 2}, Decoder)\n      |> child({:cutter, 2}, %VideoCutter{intervals: [{Membrane.Time.seconds(5), :infinity}]})\n      |> via_in(Pad.ref(:input, 2))\n      |> get_child(:merger),\n    ]\n\n    {[spec: structure], %{}}\n  end\nend\n```","ref":"readme.html#videocutter-and-videomerger"},{"type":"extras","title":"Copyright and License - Membrane VideoMerger Plugin","doc":"Copyright 2021, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane_video_merger_plugin)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=membrane_video_merger_plugin)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2020 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown"}